/*
 * Initial Developer's Public License.
 * The contents of this file are subject to the  Initial Developer's Public
 * License Version 1.0 (the "License"). You may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *   http://www.ibphoenix.com?a=ibphoenix&page=ibp_idpl
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is copyright 2004-2005 Paul Reeves.
 *
 * The Initial Developer of the Original Code is Paul Reeves
 *
 * All Rights Reserved.
 *
 */

/*
 * This script demonstrates a technique to alter all tables in a single hit.
 * The particular example adds simple auditing to every table.
 */

-- Make sure we are not still trying to write to a file.
output;

-- Make sure old stuff isn't lying around
-- ISQL has a habit of appending to files, not overwriting them.
shell del autogen_alter_all.ddl;
shell del alter_all.txt;

-- This is one of the clever bits
-- it transforms the output style into a single line
set list on;

-- This is the other clever bit. It prevents echoing the
-- executed command to file.
set echo off;

-- Redirect output to file.
output autogen_alter_all.ddl;

-- Do the real work here
select trim(RDB$RELATION_NAME) ||
 ' add INSERTED_ON TIMESTAMP default ''now'' NOT NULL ' ||
 ', add INSERTED_BY VARCHAR(31) default CURRENT_USER NOT NULL ' ||
 ', add MODIFIED_ON TIMESTAMP default ''now'' NOT NULL ' ||
 ', add MODIFIED_BY VARCHAR(31) default CURRENT_USER NOT NULL ' ||
 '; '
as "Alter table "
from RDB$RELATIONS
where RDB$RELATION_NAME not starting with 'RDB$';

select ' set term ^;' from rdb$database as "";
set term ^;
select 'tub_' || trim(RDB$RELATION_NAME) || ' for ' ||
 trim(RDB$RELATION_NAME) ||
 ' active before update position 32000 as ' ||
 ' begin ' ||
 ' New.MODIFIED_BY = CURRENT_USER; ' ||
 ' New.MODIFIED_ON = ''now''; ' ||
 ' if (new.INSERTED_ON != old.INSERTED_ON) then new.INSERTED_ON = old.INSERTED_ON; ' ||
 ' if (new.INSERTED_BY != old.INSERTED_BY) then new.INSERTED_BY = old.INSERTED_BY; ' ||
 ' end^ '

as "create trigger "
from RDB$RELATIONS
where RDB$RELATION_NAME not starting with 'RDB$'^

set term ;^
select ' set term ;^' from rdb$database as "";

-- uncomment the next line if running interactively.
--set echo on;

-- Set list to normal
set list off;

-- redirect output to our log file
output alter_all.txt;

-- make command execution visible again.
set echo on;

-- Execute our freshly minted script.
input autogen_alter_all.ddl;

--reset output to console
output;

-- Step back and admire our work
show table revisions;
show triggers;


