/*
 * Initial Developer's Public License.
 * The contents of this file are subject to the  Initial Developer's Public
 * License Version 1.0 (the "License"). You may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *   http://www.ibphoenix.com?a=ibphoenix&page=ibp_idpl
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is copyright 2004-2005 Paul Reeves.
 *
 * The Initial Developer of the Original Code is Paul Reeves
 *
 * All Rights Reserved.
 *
 */

/*
 *  Purpose:
 *    This script provides the autoinc_gen procedure. It can be used to
 *    dynamically generate triggers that set generators.
 *
 *
 *  Note:
 *    This script must be executed prior to creating tables.
 *
 *
 *  Usage:
 *		First, make this procedure available to the database.
 * 		Each table that requires a generator uses the following
 *		script snippet:
 *
 *		CREATE TABLE test_autoinc (
 *			ASequence Integer,
 *			Description Varchar(255)
 *			PRIMARY KEY  (ASequence)
 *		);
 *
 *		create sequence ASequence_seq;
 *		execute procedure autoinc_gen ('test_autoinc', 'ASequence', 'ASequence_seq');
 *		commit;
 *
 *
 *	Warning:
 *		Using EXECUTE STATEMENT with DDL can be dangerous if COMMIT
 *		is not called immediately after calling the procedure that
 *		calls EXECUTE STATEMENT.
 *
 */

--Dynamically create trigger that fires a generator on a primary key column
set term ^;
create procedure autoinc_gen (TABLE_NAME varchar(27), COLUMN_NAME varchar(31), SEQUENCE_NAME varchar(31) )
as
declare variable trig_name varchar(31);
declare variable sql varchar(1024);
begin

trig_name='tib_' || :TABLE_NAME;

sql = 'create trigger ' || :trig_name || ' FOR ' || :TABLE_NAME ||
    ' BEFORE INSERT POSITION 0 AS ' ||
	' BEGIN ' ||
	'   IF (( new.' || :COLUMN_NAME || ' is null ) or ( new.' || :COLUMN_NAME || '=0)) then ' ||
	'new.' || :COLUMN_NAME || '= gen_id(' || :SEQUENCE_NAME || ',1); ' ||
	' END'
	;

execute statement sql;

/*
 *  If we wanted to be really clever we could probably dynamically create
 *  a trigger description here using the new COMMENT ON syntax of Fb 2.0
 *
 */

end^
set term ;^
commit;

